package com.virtualmachinery.btreedemo;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.util.Random;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.UIManager;

import com.virtualmachinery.btree.demo.BTreeRO;
import com.virtualmachinery.btree.demointerfaces.BTreeROInterface;
import com.virtualmachinery.btree.exceptions.BTreeException;



public class BTreeAddressDemo extends JFrame {

	JPanel demoContentPane  = null;
	JTextField performanceText = new JTextField();
	JTextField numberEntry = new JTextField();
	JTextField addressPane = new JTextField();
	JTextPane addressResultPane = new JTextPane();
	JProgressBar progressBar = new JProgressBar();
	JButton runButton = new JButton("Run Test");
	JButton searchButton = new JButton("Search");
	JButton exitButton = new JButton("Exit");
	BTreeAddressDemoEventHandler eventHandler = new BTreeAddressDemoEventHandler();
	BTreeROInterface addressBtree = null;
	JCheckBox cacheSwitch = new JCheckBox();
	boolean cache = false;
	

	class BTreeAddressDemoEventHandler implements java.awt.event.ActionListener {
		public void actionPerformed(java.awt.event.ActionEvent e) {
			if (e.getSource() == BTreeAddressDemo.this.exitButton) 
				BTreeAddressDemo.this.dispose();
			if (e.getSource() == BTreeAddressDemo.this.runButton) 
				BTreeAddressDemo.this.runTest();
			if (e.getSource() == BTreeAddressDemo.this.searchButton) 
				BTreeAddressDemo.this.search();
			if (e.getSource() == BTreeAddressDemo.this.cacheSwitch) 
				BTreeAddressDemo.this.cacheSwitched();
		};
		
	};

	public static void main(String[] args) {
		try {
			/* Set native look and feel */
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			/* Create the frame */
			BTreeAddressDemo aBTreeDemo = new BTreeAddressDemo();

			/* Calculate the screen size */
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

			/* Pack frame on the screen */
			aBTreeDemo.pack();

			/* Center frame on the screen */
			Dimension frameSize = aBTreeDemo.getSize();
			if (frameSize.height > screenSize.height)
					frameSize.height = screenSize.height;
			if (frameSize.width > screenSize.width)
					frameSize.width = screenSize.width;
			aBTreeDemo.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

			/* Add a windowListener for the windowClosedEvent */
			aBTreeDemo.addWindowListener(new java.awt.event.WindowAdapter() {
				public void windowClosed(java.awt.event.WindowEvent e) {
					System.exit(0);
				};
			});
			aBTreeDemo.setVisible(true);
		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of BTreeDemo");
			exception.printStackTrace(System.out);
		}
	}


	public BTreeAddressDemo() {
		super();
		initialize();
	}

	// Start up
	public void initialize() {
		try {
			setName("BTreeAddressDemo");
			setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
			setSize(446, 485);
			setTitle("Virtual Machinery BTree Address Search Demo");
			addressBtree = BTreeRO.openExistingTree("VMPHONETEST");
			setContentPane(getAddressDemoContentPane());
			setVisible(true);
		} catch (java.lang.Throwable e) {
			e.printStackTrace();
		}
	}

	// Get the content pane
	private Container getAddressDemoContentPane() {
		if (demoContentPane == null) {
			demoContentPane = new javax.swing.JPanel();
			demoContentPane.setLayout(new GridBagLayout());
			demoContentPane.setName("JFrameContentPane");
			demoContentPane.setPreferredSize(new java.awt.Dimension(446, 485));
			demoContentPane.setMinimumSize(new java.awt.Dimension(446, 485));
			demoContentPane.add(getLabel("BTree Address Search Demo",true), createStandardWeightedConstraints(0,0));
			demoContentPane.add(getLabel("Search for Address",false), createStandardWeightedConstraints(0,1,1));
			demoContentPane.add(addressPane, createStandardWeightedConstraints(1,1,2));
			demoContentPane.add(addressResultPane, createStandardWeightedConstraints(0,2));
			searchButton.addActionListener(eventHandler);
			demoContentPane.add(searchButton, createStandardWeightedConstraints(1,5,1));
			demoContentPane.add(getLabel("Performance - enter number of searches in field below",true), createStandardWeightedConstraints(0,6));
			demoContentPane.add(numberEntry, createStandardWeightedConstraints(0,8,1));
			demoContentPane.add(getLabel("Use cache",false), createStandardWeightedConstraints(1,8,1));
			cacheSwitch.addActionListener(eventHandler);
			demoContentPane.add(cacheSwitch, createStandardWeightedConstraints(2,8,1));
			
			
			runButton.addActionListener(eventHandler);
			demoContentPane.add(runButton, createStandardWeightedConstraints(1,10,1));
			demoContentPane.add(progressBar, createStandardWeightedConstraints(0,12));
			demoContentPane.add(performanceText, createStandardWeightedConstraints(0,14));
			exitButton.addActionListener(eventHandler);
			demoContentPane.add(exitButton, createStandardWeightedConstraints(0,16));
			demoContentPane.setVisible(true);
		}
		return demoContentPane;
	}

	public JLabel getLabel(String text, boolean big) {
		JLabel aLabel = new JLabel();
		if (big) {
			aLabel.setFont(new java.awt.Font("Arial", Font.BOLD, 12));
			aLabel.setForeground(java.awt.Color.black);
		}
		aLabel.setText(text);
		return aLabel;
	}

	//The value of the cache has changed so reopen the B+Tree
	public void cacheSwitched() {
		cache = !cache;
		try {
			if (cache) {
				addressBtree = BTreeRO.openExistingTree("VMPHONETEST",3,132);
			} else {
				addressBtree = BTreeRO.openExistingTree("VMPHONETEST");
			}
		} catch (BTreeException e) {
			e.printStackTrace();
		}
	}
	
	// Search for the account associated with a key
	public void search() {
		String key = addressPane.getText().toString();
		String result = null;
        try {
			result = addressBtree.get(key);
		} catch (BTreeException e) {
			e.printStackTrace();
		}
        if (result !=null) {
        	addressResultPane.setText(result);
        } else {
        	addressResultPane.setText("No Details found");
        }
	}

	public void runTest () {
		performanceText.setText("Running...");
		final Thread testThread = new Thread() {
           		public void run() {
           		int count=0;
           		int entries = Integer.parseInt(numberEntry.getText().toString());
           		int start=1111111;
               	Random random = new Random();
               	int max=10000;
               	int randNo = -1;
              	long startTime = System.currentTimeMillis();
            	try { 
            		for (int i=0;i<entries;i++) {
                		randNo = ((int) (random.nextFloat()*max)) + start;
                		if (addressBtree.get(randNo+"") != null) {
						        count++;
						}
						if (count != 0) {
							final int progress = (int) ((count/(entries *1.0))*100);
                					progressBar.setValue(progress);
                				}
                			}
                	} catch (BTreeException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} finally {
				if (count != entries) count = -1*(count-entries);
				String resultString = count+" Entries took "+(System.currentTimeMillis() - startTime)+" ms";
				performanceText.setText(resultString);
        		progressBar.setValue(0);
			}
            	
          }
       };
       testThread.start();   	
	}

	public static GridBagConstraints createStandardWeightedConstraints(int xValue, int yValue) {
		return createStandardWeightedConstraints( xValue, yValue,3);
	}
	/**
	 * Create Standard constraints
	 * @param xValue
	 * @param yValue
	 * @return
	 */
	public static GridBagConstraints createStandardWeightedConstraints(int xValue, int yValue, int width) {
		java.awt.GridBagConstraints standard = new java.awt.GridBagConstraints();
		standard.gridx = xValue; 
		standard.gridy = yValue;
		standard.gridwidth = width;
		standard.fill = java.awt.GridBagConstraints.HORIZONTAL;
		standard.anchor = java.awt.GridBagConstraints.NORTHWEST;
		standard.weightx = 1.0;
		standard.weighty = 1.0;
		standard.ipadx = 12;
		standard.ipady = 12;
		standard.insets = new java.awt.Insets(5, 5, 5, 5);
		return standard;
	}
	
}
