package com.virtualmachinery.btreedemo;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.LineUnavailableException;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.UIManager;

import com.virtualmachinery.btree.demo.BTreeRO;
import com.virtualmachinery.btree.demo.BTreeROFactory;
import com.virtualmachinery.btree.demointerfaces.BTreeROInterface;
import com.virtualmachinery.btree.exceptions.BTreeException;


public class BTreeMediaDemo extends JFrame {

	JPanel demoContentPane = null;
	JTabbedPane tabPane = null;
	JButton exitButton = null;
	JButton soundButton = null;
	BTreeMediaDemoEventHandler eventHandler = new BTreeMediaDemoEventHandler();
	BTreeROInterface mediaBtree = null;
	byte[] sound;
	
	Dimension PREFERRED_SIZE = new java.awt.Dimension(350, 485);
	
	//Handle the button click events
	class BTreeMediaDemoEventHandler implements java.awt.event.ActionListener {
		public void actionPerformed(java.awt.event.ActionEvent e) {
			if (e.getSource() == BTreeMediaDemo.this.exitButton) 
				BTreeMediaDemo.this.dispose();
			if (e.getSource() == BTreeMediaDemo.this.soundButton) 
				BTreeMediaDemo.this.playSound();
		};	
	};
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			/* Set native look and feel */
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			/* Create the frame */
			BTreeMediaDemo aBTreeDemo = new BTreeMediaDemo();

			/* Calculate the screen size */
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

			/* Pack frame on the screen */
			aBTreeDemo.pack();

			/* Center frame on the screen */
			Dimension frameSize = aBTreeDemo.getSize();
			if (frameSize.height > screenSize.height)
					frameSize.height = screenSize.height;
			if (frameSize.width > screenSize.width)
					frameSize.width = screenSize.width;
			aBTreeDemo.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

			/* Add a windowListener for the windowClosedEvent */
			aBTreeDemo.addWindowListener(new java.awt.event.WindowAdapter() {
				public void windowClosed(java.awt.event.WindowEvent e) {
					System.exit(0);
				};
			});
			aBTreeDemo.setVisible(true);
		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of BTreeDemo");
			exception.printStackTrace(System.out);
		}
	}
	
	/**
	 * BTreeMediaDemo constructor comment.
	 */
	public BTreeMediaDemo() {
		super();
		initialize();
	}
	
	/**
	 * Initialize the class.
	 */
	private void initialize() {
		try {
			setName("BTreeDemo");
			setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
			setSize(PREFERRED_SIZE);
			setTitle("Virtual Machinery BTree Media Demo");
			mediaBtree = BTreeRO.openExistingTree("MEDIA");
			setContentPane(getMediaDemoContentPane());
			setVisible(true);
		} catch (java.lang.Throwable t) {
			t.printStackTrace();
		}
	}

	// main pane
	private Container getMediaDemoContentPane() {
		if (demoContentPane == null) {
			demoContentPane = new javax.swing.JPanel();
			demoContentPane.setName("JFrameContentPane");
			demoContentPane.setPreferredSize(PREFERRED_SIZE);
			demoContentPane.setLayout(new java.awt.BorderLayout());
			demoContentPane.setMinimumSize(PREFERRED_SIZE);
			demoContentPane.add(getButtonPane(), "South");
			demoContentPane.add(getDemoPane(), "Center");
		}
		return demoContentPane;
	}

	/**
	 * Create the tabbed pane.
	 * @return javax.swing.JTabbedPane
	 */
	private javax.swing.JTabbedPane getDemoPane() {
		if (tabPane == null) {
			tabPane = new javax.swing.JTabbedPane();
			tabPane.setName("MainPane");
			tabPane.insertTab("Text Demo", null, getTextPanel(), null, 0);
			tabPane.insertTab("Image Demo", null, getImagePanel(), null, 1);
			tabPane.insertTab("Sound Demo", null, getSoundPanel(), null, 2);
		}
		return tabPane;
	}
	
	// Create the text panel populating the text area with text loaded from the B+Tree
	private JPanel getTextPanel() {
		JPanel textPanel = new JPanel();
		textPanel.setLayout(new GridBagLayout());
		JTextArea tPane = this.getTextAreaFromKey("TEXT_KEY");
		tPane.setLineWrap(true);
		GridBagConstraints constr = new GridBagConstraints();
		constr.anchor=GridBagConstraints.NORTHWEST;
		constr.gridwidth=1;
		constr.gridheight=1;
		constr.weightx=1.0;
		constr.weighty=1.0;
		constr.fill=GridBagConstraints.BOTH;
		textPanel.add(tPane,constr);
		return textPanel;
	}
	
	// Create the image panel populating the text area with text loaded from the B+Tree
	// and the image area with an image loaded from the B+Tree
	private JPanel getImagePanel() {
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new GridBagLayout());
	
		JTextArea tPane = this.getTextAreaFromKey("IMAGE_TEXT");
		GridBagConstraints constr2 = new GridBagConstraints();
		tPane.setLineWrap(true);
		constr2.gridx=0;
		constr2.gridy=0;
		constr2.gridwidth=4;
		constr2.gridheight=1;
		constr2.weightx=1.0;
		constr2.weighty=0.0;
		constr2.fill=GridBagConstraints.HORIZONTAL;
		mainPanel.add(tPane,constr2);
		
		ImagePanel imagePanel = new ImagePanel();
		BufferedImage image = null;
		try {
			image = restoreImage("PICTURE_KEY");
		} catch (IOException e) {
			e.printStackTrace();
		}
		if (image != null) imagePanel.setBufferedImage(image);
		GridBagConstraints constr = new GridBagConstraints();
		constr.gridx=0;
		constr.gridy=1;
		constr.gridwidth=4;
		constr.gridheight=6;
		constr.weightx=1.0;
		constr.weighty=1.0;
		constr.ipadx=5;
		constr.ipady=5;
		constr.fill=GridBagConstraints.BOTH;
		mainPanel.add(imagePanel,constr);
	
		return mainPanel;
	}
	
	// Create the Sound Panel, populating the text area from the B+Tree
	private JPanel getSoundPanel() {
		JPanel soundPanel = new JPanel();
		soundPanel.setLayout(new GridBagLayout());
		JTextArea tPane = this.getTextAreaFromKey("SOUND_TEXT");
		tPane.setLineWrap(true);
		GridBagConstraints constr = new GridBagConstraints();
		constr.anchor =GridBagConstraints.NORTHWEST;
		constr.gridx=0;
		constr.gridy=0;
		constr.gridwidth=3;
		constr.gridheight=5;
		constr.weightx=1.0;
		constr.weighty=1.0;
		constr.fill=GridBagConstraints.BOTH;
		soundPanel.add(tPane,constr);
		sound = getBytesFromTree("SOUNDWAV_KEY");
		soundButton = new JButton("Play Sound");
		soundButton.addActionListener(eventHandler);
		GridBagConstraints constr2 = new GridBagConstraints();
		constr2.anchor =GridBagConstraints.SOUTH;
		constr2.gridx=2;
		constr2.gridy=6;
		constr2.gridwidth=1;
		constr2.gridheight=1;
		constr2.fill=GridBagConstraints.NONE;
		soundPanel.add(soundButton,constr2);
		return soundPanel;
	}
	
	//Play the sound loaded fom the B+Tree
	private void playSound() {
        ByteArrayInputStream bais = new ByteArrayInputStream(sound);
        AudioInputStream ais = new AudioInputStream(bais, new AudioFormat(44100f, 16, 1, true, false), sound.length);
        Clip clip;
		try {
			clip = AudioSystem.getClip();
	        clip.open(ais);
	        clip.start();
	     } catch (LineUnavailableException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
	
	//Populate the text area with the text that is located in the B+Tree indexed by the key
	public JTextArea getTextAreaFromKey(String key) {
		JTextArea tPane = new JTextArea();
		tPane.setFont(new Font(Font.SANS_SERIF,Font.BOLD,16));
		tPane.setLineWrap(true);
		tPane.setWrapStyleWord(true);
		try {
			tPane.setText(mediaBtree.get(key));
		} catch (BTreeException e) {
			e.printStackTrace();
		}
		return tPane;
	}
	
	//Creates an image from the bytes in the B+Tree
	public BufferedImage restoreImage(String index) throws IOException {
		byte[] pixels = getBytesFromTree(index);
		ByteArrayInputStream bs = new ByteArrayInputStream(pixels);
		return ImageIO.read(ImageIO.createImageInputStream(bs));
	}
	
	//Load the bytes from the B+Tree
	//This uses the approach where the base key is a string that has the number of sub_records and the number of bytes separated by a '^'
	// Each sub_record has an index made up of the base key plus the underscore ('_') symbol plus the record number
	// As each sub_record is fetched the bytes are placed into the array.
	public byte[] getBytesFromTree(String index) {
		String iString;
		try {
			iString = (String) mediaBtree.get(index);
			if (iString != null) {
				String[] base = ((String) mediaBtree.get(index)).split("\\^");
				int numRecords = Integer.parseInt(base[0]);
				int pixLength = Integer.parseInt(base[1]);
				byte[] pixels = new byte[pixLength];
				int start = 0;
				for (int i=0;i<numRecords;i++) {
					byte[] thisRec = mediaBtree.get(BTreeROFactory.getInstance().createRecord(index+"_"+i)).getValue();
					System.arraycopy(thisRec, 0, pixels, start, thisRec.length);
					start+=thisRec.length;
				}
				return pixels;
			}
			else {
				return null;
			}
		} catch (BTreeException e) {
			e.printStackTrace();
		}
	return null;
	}
	
	// Create the exit button panel
	private Component getButtonPane() {
		JPanel buttonPanel = new JPanel();
		exitButton = new JButton("Exit");
		exitButton.addActionListener(eventHandler);
		buttonPanel.add(exitButton);
		return buttonPanel;
	}

}
